# -*- coding: utf-8 -*-

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from jmb.core.admin.options import ExtendibleModelAdmin
from django.conf.urls import patterns, url

from acomea.my_acomea.models import FileGestioni
from acomea.my_acomea.admin_views import FileGestioniImportView


class FileGestioniAdmin(ExtendibleModelAdmin):
    advanced_search_fields = (
        ('rapporto__icontains', 'name__icontains'),
        ('date__gte', 'date__lte'),
    )
    list_display = ('get_edit_icon', 'date', 'rapporto', 'name', 'pdf_link', 'get_delete_icon',)
    list_display_csv = ('date', 'rapporto', 'name')
    date_hierarchy = 'date'
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'date', 'name', 'rapporto', 'file',
            ),
        }),
    )

    def pdf_link(self, instance):
        if instance.file:
            return u"<a href='{}' target='_blank'>Pdf</a>".format(instance.file.url)
        return u""
    pdf_link.short_description = _("File")
    pdf_link.allow_tags = True

    def get_urls(self):
        urls = super(FileGestioniAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^upload/$',
                self.admin_site.admin_view(FileGestioniImportView.as_view()),
                name='my_acomea_filegestioni_import'
            ),
        )
        return my_urls + urls

    def get_actions(self, request):
        actions = super(FileGestioniAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions


admin.site.register(FileGestioni, FileGestioniAdmin)
