# -*- coding: utf-8 -*-

from cms.models.pluginmodel import CMSPlugin
from colorful.fields import RGBColorField
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext as _

from jmb.core.db.cmsmodels import StatusModel
from jmb.core.db.models import OrderedModel
from jmb.core.db.fields import HTMLFieldTinyMCE


class MyAcomeAPluginSettings(CMSPlugin):
    display = models.CharField(
        max_length=100,
        choices=settings.ACOMEA_MYACOMEA_PLUGIN_DISPLAY_CHOICES,
    )

    class Meta:
        app_label = "my_acomea"

    def __unicode__(self):
        return u"{}".format(settings.ACOMEA_MYACOMEA_PLUGIN_SETTINGS[self.display]['name'])


class MyAcomeAccordionSettings(CMSPlugin, StatusModel):
    title = models.CharField(
        max_length=255,
        verbose_name=_('Title'),
    )
    PRODUCT = (
        ('FC', 'Fondi comuni'),
        ('G', 'Gestioni'),
        ('D', 'User is DEAD'),
    )
    category = models.CharField(
        max_length=255,
        verbose_name=_('category'),
        choices=PRODUCT
    )
    disable_if_not_user = models.BooleanField(
        default=False,
        verbose_name=_('disable if not user'),
    )
    disable_if_not_lead = models.BooleanField(
        default=False,
        verbose_name=_('disable if not customer'),
    )
    template_name = models.CharField(
        _("Template"),
        max_length=150,
        choices=settings.ACOMEA_MY_ACOMEA_ACCORDION_PLUGIN_TEMPLATE_CHOICES,
        default="accordion/cms_plugins/accordion.html",
    )

    class Meta:
        app_label = "my_acomea"

    def __unicode__(self):
        return u"{}".format(self.title)


class MyAcomeAccordionItemSettings(OrderedModel):
    plugin = models.ForeignKey(
        MyAcomeAccordionSettings,
        related_name="items"
    )
    title = models.CharField(
        max_length=255,
        verbose_name=_('Title'),
    )
    text_color = RGBColorField(
        _("text color"),
    )
    page_slug = models.SlugField(
        _("Page slug"),
        max_length=100,
        blank=True, null=True,
    )

    class Meta:
        app_label = "my_acomea"
        ordering = ("ordering",)

    def __unicode__(self):
        return u'%s: %s' % (self.plugin.title, self.title)


class MyAcomeAModalSettings(CMSPlugin, StatusModel):
    title = models.CharField(
        max_length=100,
        verbose_name=_('Title'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
    )

    class Meta:
        app_label = "my_acomea"

    def __unicode__(self):
        return u'%s (%s)' % (self.title, "on" if self.status else "off")
