# -*- coding: utf-8 -*-

import datetime
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _

from jmb.core.middleware.force_response import ForceResponse
from jmb.core.utils.cmsutils import get_abs_url_from_slug

from acomea.areariservata.soap import AcomeaClientRiservataCms
from acomea.my_acomea.cms.models import (
    MyAcomeAPluginSettings, MyAcomeAccordionSettings, MyAcomeAccordionItemSettings, MyAcomeAModalSettings
)
from acomea.my_acomea.forms import AnalisiTemporaleForm
from acomea.profilo.cms.plugins import _get_cambia_posizione
from acomea.profilo.models import RichiestaAttivazioneCliente
from acomea.soap2.soap import MyAcomeaSoapClient


class MyAcomeAPluginMixin(object):

    def __init__(self, *args, **kwargs):
        self.request = None
        self.user = None
        self.codice_fiscale = None
        self.user_cliente = None
        self.profilo_cliente = None
        self.rac_cliente = None
        self.cliente_soap = None
        self.is_dead = False
        self.must_compile_mifid = False
        self.is_enabled = False
        super(MyAcomeAPluginMixin, self).__init__(*args, **kwargs)

    def get_mifid_result(self):
        soap_areariservata = AcomeaClientRiservataCms()
        # TODO: teoricamente questo if si può rimuovere
        if self.codice_fiscale == None:
            self.codice_fiscale = ''
        return soap_areariservata.qst_must_compile_ar_by_sott(self.codice_fiscale)

    def get_or_set_mifid_in_session(self):
        result = self.get_mifid_result()
        self.request.session['must_compile_mifid'] = False
        if result['ISOK']:
            if result['MCM_Antiriciclaggio']:
                self.request.session['must_compile_mifid'] = 'ANTIRICICLAGGIO'
            elif result['MCM_Appropriatezza']:
                self.request.session['must_compile_mifid'] = 'APPROPRIATEZZA'
            elif result['MCM_Adeguatezza']:
                self.request.session['must_compile_mifid'] = 'ADEGUATEZZA'
        return self.request.session['must_compile_mifid']

    def get_must_compile_mifid(self):
        if (
            self.profilo_cliente.is_cliente
            and self.rac_cliente.stato in ('6', '9')
        ):
            must_compile_mifid = self.get_or_set_mifid_in_session()
            # verifica del mifid
            if must_compile_mifid:
                return True
            return False
        self.request.session['must_compile_mifid'] = False
        return False

    def get_codice_fiscale(self):
        if 'codice_fiscale' in self.request.POST:
            cf = self.request.POST.get('codice_fiscale')
        elif 'codice_fiscale' in self.request.session:
            cf = self.request.session['codice_fiscale']
        elif (
            self.user.is_active and hasattr(self.user, 'profilo') and
            hasattr(self.user.profilo, 'rac')
        ):
            cf = self.user.profilo.rac.codice_fiscale
        else:
            cf = ''
        if cf:
            self.request.session['codice_fiscale'] = cf
        return cf.upper()

    def get_cliente(self):
        cliente = dict()
        if (
            'cliente_my_acomea' in self.request.session and
            self.request.session['cliente_my_acomea']['codice_fiscale'] == self.codice_fiscale
        ):
            cliente = self.request.session['cliente_my_acomea']
        else:
            soap_client = MyAcomeaSoapClient()
            cliente_soap = soap_client.get_cliente(self.codice_fiscale)
            if cliente_soap['codice_fiscale']:
                self.request.session['cliente_my_acomea'] = cliente_soap
                cliente = cliente_soap
            else:
                if 'codice_fiscale' in self.request.session:
                    del self.request.session['codice_fiscale']
                if 'cliente_my_acomea' in self.request.session:
                    del self.request.session['cliente_my_acomea']
                self.request.session.modified = True
        return cliente

    def get_is_dead(self):
        if 'is_dead' in self.cliente_soap:
            return self.cliente_soap['is_dead']
        return False

    def get_rac_cliente(self):
        try:
            return RichiestaAttivazioneCliente.objects.get(codice_fiscale=self.codice_fiscale)
        except RichiestaAttivazioneCliente.DoesNotExist:
            return ''

    def get_profilo_cliente(self):
        if self.rac_cliente and hasattr(self.rac_cliente, 'profilo'):
            return self.rac_cliente.profilo
        else:
            return ''

    def get_user_cliente(self):
        if self.profilo_cliente and hasattr(self.profilo_cliente, 'user'):
            return self.profilo_cliente.user
        else:
            return ''

    @property
    def is_test(self):
        if self.profilo_cliente and self.profilo_cliente.is_acomea:
            return True
        return False

    def get_is_enabled(self):
        if self.profilo_cliente.is_acomea:
            return True
        if self.profilo_cliente and self.rac_cliente:
            if (
                self.profilo_cliente.is_cliente and self.profilo_cliente.abilitato and
                self.rac_cliente.stato in ('9', '6') and self.rac_cliente.codice_fiscale
            ):
                return True
        return False

    def init_cliente(self):
        try:
            self.user = self.request.user
            if self.user.is_authenticated():
                self.codice_fiscale = self.get_codice_fiscale()
                self.cliente_soap = self.get_cliente()
                self.is_dead = self.get_is_dead()
                self.rac_cliente = self.get_rac_cliente()
                self.profilo_cliente = self.get_profilo_cliente()
                self.user_cliente = self.get_user_cliente()
                self.must_compile_mifid = self.get_must_compile_mifid()
                self.is_enabled = self.get_is_enabled()
        except Exception as e:
            print e


class MyAcomeAPlugin(MyAcomeAPluginMixin, CMSPluginBase):
    module = _('My AcomeA')
    model = MyAcomeAPluginSettings
    name = _('My AcomeA Plugin')
    admin_preview = False
    render_template = "my_acomea/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        self.request = context['request']
        self.init_cliente()
        # TEST
        # DATI UTENTE COLLEGATO
        if self.is_test:
            test = dict()

            # DATI CLIENTE
            test['codice_fiscale'] = self.codice_fiscale
            test['cliente'] = self.cliente_soap
            test['user_cliente'] = self.user_cliente
            test['profilo_cliente'] = self.profilo_cliente
            test['rac_cliente'] = self.rac_cliente
            test['must_compile_mifid'] = self.must_compile_mifid
            test['is_dead'] = self.is_dead
            test['is_enabled'] = self.is_enabled
            context['test'] = test

            # self.render_template = "my_acomea/cms_plugins/test_myacomeaplugin.html"
            # return context

        if self.request.user.is_authenticated():
            # FORZATO REDIRECT A NUOVA MYACOMEA
            # TODO utente is_dead
            raise ForceResponse(HttpResponseRedirect(settings.ACOMEA_MY_ACOMEA_URL_REDIRECT_ANGJS_MY_ACOMEA))

        # # Redirect se utente acomea v.sotto redirect cliente
        # if hasattr(self.user, 'profilo') and self.user.profilo.is_acomea:
        #     raise ForceResponse(HttpResponseRedirect(u'https://myacomea.acomea.it'))

        if not self.codice_fiscale or not self.cliente_soap:
            return context

        if self.is_dead:
            context.update({'is_dead': True})
            self.render_template = "my_acomea/cms_plugins/is_dead.html"
            return context

        if not self.rac_cliente or not self.profilo_cliente or not self.user_cliente:
            if hasattr(self.user, 'profilo') and self.user.profilo.is_acomea:
                if hasattr(self.user.profilo, 'rac'):
                    messages.add_message(
                        self.request, messages.WARNING,
                        'ATTENZIONE: CF privo di RAC, simulazione ottenuta usando RAC AcomeA'
                    )
                    self.rac_cliente = self.user.profilo.rac
                    self.is_enabled = True
                else:
                    return context
            else:
                return context

        if self.must_compile_mifid:
            return context

        # if self.request.user.username == 'fabiopre' and self.rac_cliente.stato in ('6', '9'):
        # Redirect se cliente v.sopra redirect utente acomea
        # if self.rac_cliente.stato in ('6', '9'):
        #     raise ForceResponse(HttpResponseRedirect(u'https://myacomea.acomea.it'))

        if not self.is_enabled:
            if instance.display == 'my_acomea':
                return context
            raise ForceResponse(HttpResponseRedirect('/my_acomea/'))

        self.render_template = settings.ACOMEA_MYACOMEA_PLUGIN_SETTINGS[instance.display]['template']

        self.render_template = settings.ACOMEA_MYACOMEA_PLUGIN_SETTINGS[instance.display]['template']

        if instance.display == 'my_acomea':
            context.update({
                'cambia_posizione': _get_cambia_posizione(
                    self.request, self.user_cliente, self.rac_cliente.stato),
                'url_intestatario':  reverse(
                    'myacomea:my_acomea', kwargs={'partecipazione': 'INTESTATARIO'}
                ),
                'url_cointestatario': reverse(
                    'myacomea:my_acomea', kwargs={'partecipazione': 'COINTESTATARIO'}
                ),
            })

        elif instance.display == "fondi_dettaglio":
            context.update({
                'cambia_posizione': _get_cambia_posizione(
                    self.request, self.user_cliente, self.rac_cliente.stato),
                'url_intestatario':  reverse(
                    'myacomea:fondi_dettaglio', kwargs={'partecipazione': 'INTESTATARIO'}
                ),
                'url_cointestatario': reverse(
                    'myacomea:fondi_dettaglio', kwargs={'partecipazione': 'COINTESTATARIO'}
                ),
            })

        elif instance.display == "fondi_analisi_temporale":
            url_submit = reverse('myacomea:analisi_temporale')
            context.update({
                'form': AnalisiTemporaleForm(),
                'url_submit': url_submit,
            })

        elif instance.display == 'fondi_operazioni_in_corso':
            context.update({'url': reverse('myacomea:fondi_operazioni_in_corso')})

        elif instance.display == 'fondi_operazioni_effettuate':
            context.update({
                'url_intestatario':  reverse(
                    'myacomea:fondi_operazioni_effettuate', kwargs={'partecipazione': 'INTESTATARIO'}
                ),
                'url_cointestatario': reverse(
                    'myacomea:fondi_operazioni_effettuate', kwargs={'partecipazione': 'COINTESTATARIO'}
                ),
            })

        elif instance.display == 'fondi_servizi_attivi':
            context.update({'url': reverse('myacomea:fondi_servizi_attivi')})

        elif instance.display == 'gestioni_operazioni_effettuate':
            context.update({
                'url_intestatario':  reverse(
                    'myacomea:gestioni_operazioni_effettuate', kwargs={'partecipazione': 'INTESTATARIO'}
                ),
                'url_cointestatario': reverse(
                    'myacomea:gestioni_operazioni_effettuate', kwargs={'partecipazione': 'COINTESTATARIO'}
                ),
            })

        elif instance.display == 'gestioni_dettaglio':
            context.update({
                'cambia_posizione': _get_cambia_posizione(
                    self.request, self.user_cliente, self.rac_cliente.stato),
                'url_intestatario':  reverse(
                    'myacomea:gestioni_dettaglio', kwargs={'partecipazione': 'INTESTATARIO'}
                ),
                'url_cointestatario': reverse(
                    'myacomea:gestioni_dettaglio', kwargs={'partecipazione': 'COINTESTATARIO'}
                ),
            })

        elif instance.display == 'gestioni_riclassificazione_portafogli':
            context.update({
                'url_intestatario':  reverse(
                    'myacomea:gestioni_riclassificazione_portafogli', kwargs={'partecipazione': 'INTESTATARIO'}
                ),
                'url_cointestatario': reverse(
                    'myacomea:gestioni_riclassificazione_portafogli', kwargs={'partecipazione': 'COINTESTATARIO'}
                ),
            })

        elif instance.display == 'gestioni_dettaglio_titoli':
            context.update({
                'url_intestatario':  reverse(
                    'myacomea:gestioni_dettaglio_titoli', kwargs={'partecipazione': 'INTESTATARIO'}
                ),
                'url_cointestatario': reverse(
                    'myacomea:gestioni_dettaglio_titoli', kwargs={'partecipazione': 'COINTESTATARIO'}
                ),
            })

        elif instance.display == 'gestioni_storico_estratto_conto':
            context.update({
                'url_intestatario':  reverse(
                    'myacomea:gestioni_storico_estratto_conto', kwargs={'partecipazione': 'INTESTATARIO'}
                ),
                'url_cointestatario': reverse(
                    'myacomea:gestioni_storico_estratto_conto', kwargs={'partecipazione': 'COINTESTATARIO'}
                ),
            })
        else:
            return context

        context.update({
            'today': datetime.date.today(),
        })
        return context


class MyAcomeAccordionItemAdmin(admin.StackedInline):
    model = MyAcomeAccordionItemSettings
    extra = 0
    fields = ('title', 'text_color', 'page_slug', 'ordering')


class MyAcomeAccordionPlugin(MyAcomeAPluginMixin, CMSPluginBase):
    model = MyAcomeAccordionSettings
    name = _("Accordion")
    module = _("My AcomeA")
    admin_preview = False
    render_template = "my_acomea/cms_plugins/empty.html"
    inlines = (MyAcomeAccordionItemAdmin,)

    def render(self, context, instance, placeholder):
        self.request = context['request']

        items = []
        is_open, visible_accordion = False, False
        self.init_cliente()
        category = instance.category
        if self.cliente_soap:
            if not self.is_enabled:
                visible_accordion = True

            if self.is_dead:
                # Dead
                if category == 'D':
                    visible_accordion = True

            if not self.is_dead and self.is_enabled:
                # Fondi Comuni
                if category == 'FC':
                    key_1 = self.cliente_soap.get('has_fondi_comuni_intestatario', False)
                    key_2 = self.cliente_soap.get('has_fondi_comuni_cointestatario', False)
                    if key_1 or key_2 is True:
                        visible_accordion = True

                # Gestioni
                if category == 'G':
                    key_1 = self.cliente_soap.get('has_gestioni_intestatario', False)
                    key_2 = self.cliente_soap.get('has_gestioni_cointestatario', False)
                    if key_1 or key_2 is True:
                        visible_accordion = True

        for item in instance.items.all().order_by('ordering'):
            url = get_abs_url_from_slug(item.page_slug) if item.page_slug else None

            is_active = True if self.request.path == url else False

            items.append({
                'title': item.title,
                'color': item.text_color,
                'url': url,
                'is_active': is_active,
                'is_disabled': not self.is_enabled
            })
            if is_active:
                is_open = True
        link_enabled = True
        if instance.disable_if_not_lead:
            if not self.user.is_authenticated():
                link_enabled = False
            if not hasattr(self.user, 'profilo'):
                link_enabled = False
            else:
                if (
                    hasattr(self.user_cliente, 'profilo') and
                    (self.user_cliente.profilo.is_only_lead or not self.user_cliente.profilo.is_lead)
                ):
                    link_enabled = False
        elif instance.disable_if_not_user:
            if not self.user.is_authenticated():
                link_enabled = False

        self.render_template = instance.template_name

        context.update({
            'instance': instance,
            'placeholder': placeholder,
            'items': items,
            'link_enabled': link_enabled,
            'is_open': is_open,
            'visible_accordion': visible_accordion,
            'must_mifid_compile': self.must_compile_mifid,
        })
        return context


class MyAcomeAModalPlugin(CMSPluginBase):
    model = MyAcomeAModalSettings
    name = _("Modal")
    module = _("My AcomeA")
    admin_preview = False
    render_template = "my_acomea/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context["request"]
        user = request.user
        cookie_key = 'my_acomea_modal_{}'.format(instance.id)
        if not instance.status:
            return context
        if user.is_authenticated() and cookie_key not in request.COOKIES:
            self.render_template = "my_acomea/cms_plugins/modal.html"
        context.update({
            'instance': instance,
            'placeholder': placeholder,
            'cookie_key': cookie_key,
            'modal_id': 'my_acomea_modal_{}'.format(instance.id),
            'close_url': reverse('myacomea:close_modal')
        })
        return context


plugin_pool.register_plugin(MyAcomeAPlugin)
plugin_pool.register_plugin(MyAcomeAccordionPlugin)
plugin_pool.register_plugin(MyAcomeAModalPlugin)
