# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.db.models.signals import post_delete
from south.modelsinspector import add_introspection_rules

from jmb.core.db.models import DateModel, UserModel
from acomea.my_acomea.signals import file_gestioni_post_delete_handler

add_introspection_rules([], ["^colorful\.fields\.RGBColorField"])


class FileGestioni(UserModel, DateModel):
    name = models.CharField(
        max_length=100,
        verbose_name=_("Name"),
    )
    date = models.DateField(
        verbose_name=_("Data rendiconto"),
    )
    rapporto = models.CharField(
        max_length=10,
        verbose_name=_("Rapporto rendiconto"),
        db_index=True,
    )
    file = models.FileField(
        verbose_name=_("File"),
        upload_to='gestioni/files/%Y/%m',
    )

    class Meta:
        ordering = ('-date', 'rapporto',)
        verbose_name = _('File gestione')
        verbose_name_plural = _('File gestioni')

    def __unicode__(self):
        return u"{}".format(self.name)
post_delete.connect(file_gestioni_post_delete_handler, sender=FileGestioni)

if settings.CMS_ENABLED:
    import acomea.my_acomea.cms.plugins
    import acomea.my_acomea.cms.app
